
#include "ByteStore.h"


int ByteStore::store(const String& string)
   {
   int str_len = string.length();
   err = resize(str_len+1);
   if (err==0)
	{
	err=next_free_index;
	for (int n=0; n<str_len; n++)
		*(start+(next_free_index++)) = string.elem(n);
	*(start+(next_free_index++)) = '\0';  //add null!
	};
   return err;
   };

int ByteStore::store(const char ch)
   {
   err = resize(1);
   if (err==0) 
	{
	err=next_free_index;
	*(start+(next_free_index++)) = ch;
	};
   return err;
   }

int ByteStore::store(const char* ch_p)
   {
   int str_len = strlen(ch_p)+1;
   err = resize(str_len);
   if (err==0)
	{
	err=next_free_index;
	for (int n=0; n<str_len; n++)
		{*(start+(next_free_index++)) = *(ch_p+n);};
	};
   return err;
   };

#ifdef _ByteStore_platform_dependant
int ByteStore::store(const int int_n)
   {
   err = resize(sizeof(int));
   if (err==0)
	{
	err=next_free_index;
	const char* int_p = (char*) &int_n;
	for (int n=1; n<=sizeof(int); n++)
	   {
	   *(start+(next_free_index++)) = *(int_p++);
	   };
	};
   return err;
   };
#else
int ByteStore::store(const int int_n)
   {
   err = resize(4);
   if (err==0)
	{
	err=next_free_index;
	int remainder, n, divisor;
	divisor = 256;
	n=int_n;
	for (int a=1; a<=4; a++)
	   {
	   remainder=n%divisor;
	   n=n/divisor;
	   store((char)remainder);
	   };
	};
   return err;
   };

#endif

void ByteStore::replace(const int int_n, const int address)
   {
   int temp=next_free_index;
   next_free_index=address;
   err=store(int_n);
   next_free_index=temp;
   };


void ByteStore::get(char& ch)
   {
   ch = *(start+(next_get_index++));
   };

void ByteStore::get(char* ch_p)
   {
   strcpy (ch_p, (start+next_get_index));
   next_get_index += (strlen (ch_p)+1);
   };

void ByteStore::get(String& string)
   {
   string = (start+next_get_index);
   next_get_index += (string.length()+1);  
   };

#ifdef _ByteStore_platform_dependant
void ByteStore::get(int& int_n)
   {
//cout << "old get method ";
   char* int_p = (char*) &int_n;
   for (int n=1; n<=sizeof(int); n++)
	{
	*(int_p++) = *(start+(next_get_index++));
	};
   };
#else
void ByteStore::get(int& int_n)
   {
//cout << "new get method ";
   int remainder, multiplier;
   multiplier = 1;
   int_n=0;
   for (int a=1; a<=4; a++)
      {
      int_n=int_n+((unsigned char)(get_char())*multiplier);
      multiplier=multiplier*256;
      };
//cout << "get int :: "<<int_n<<"\n";
   };
#endif

char ByteStore::get_char()
   {
   char ch = *(start+(next_get_index++));
   return ch;
   };

int ByteStore::get_int()
   {
   int int_n;
   get(int_n);
   return int_n;
   };

char* ByteStore::get_char_ptr()
   {
   char* ch_p = (start+next_get_index);
   next_get_index += (strlen (ch_p)+1);
   return ch_p;
   };

ByteStore::~ByteStore()  
   {
   free(start);
   };
	

int ByteStore::resize(int required_bytes) 
	{
	if (required_bytes>size-next_free_index)

		{ 
		size = required_bytes+expand_by+size;
		start=(char*) realloc(start, size);
		};

	if (start==NULL) return -1;
		else return 0;
	}; //end resize







