#include <iostream.h>
#include <stdlib.h>
#include <String.h>
#include "ss.h"

void string_set_to_string(Set_String_ set, String& string)
   {
   set.next_member_returns_first();
   while (set.can_use_get_next_member())
	cat (string, quoted(set.get_next_member()), " ", string);
   };

void resource_set_to_string(Set_Resource_ set, String& string)
   {
   Resource temp;
   set.next_member_returns_first();
   while (set.can_use_get_next_member())
	{
	temp=set.get_next_member();
	cat (string, quoted(temp.url), " ", string);
	cat (string, quoted(temp.name), " ", string);
	//cat (string, quoted(temp.description), " ", string);
	};
   };




void Set_String_::fill_with 
	(String array[], int start_index, int end_index)
   {
   for (int n=start_index; n<=end_index; n++)
	add_member(array[n]);
   };

Set_String_::Set_String_ (unsigned expand_rate_=10)
   {
   expand_rate=expand_rate_;
   size=0;
   last_member=-1;
   last_accessed_member=-1;
   object_array=NULL;   // no point in mallocing them memory yet, as the set
			// has no members, so we must NULL them so as not to
			// confuse other member functions, eg empty
   };


Set_String_::Set_String_ (const Set_String_& original)
   {
   expand_rate=original.expand_rate;
   size=0;
   last_member=-1;
   last_accessed_member=-1;
   object_array=NULL;
   operator=(original);  
   };


Set_String_::~Set_String_ ()  
   {
   for (int n=0; n<=last_member; n++)
	{
	delete(object_array[n]);   // delete memory of a set member
	};
   if (object_array!=NULL) free (object_array); //delete object pointer array
   };


void Set_String_::resize_to (unsigned newsize)
   {
   if (newsize>size)   // ie only resize if we need to
     {
     // this if is necessary as some versions of realloc 
     // don't work with NULL pointers
     if (object_array==NULL)
	object_array = (String**) 
			   malloc((newsize+expand_rate+1)*sizeof(String**));
      else
	object_array = (String**) realloc(object_array, 
				(newsize+expand_rate+1)*sizeof(String**) );
      size=newsize+expand_rate+1;
      };
   };

void Set_String_::add_member (const String& object,
				 const Add_Flag flag = TEST_FOR_DUPLICATE)
   {
   if (flag==NO_TEST_FOR_DUPLICATE)
	{
	resize_to(no_of_members()+1);   // make sure theres room
   	object_array[++last_member] = new(String); // make a new object & set up
					      // a pointer to it after the 
					      // last pointer in object_array
   	*object_array[last_member] = object; //copy the object to our new one
	}
    else
     {
     if (find_member(object)==-1)  // ie can't have duplicates in a set!
	{
	resize_to(no_of_members()+1);   // make sure theres room
   	object_array[++last_member] = new(String); // make a new object & set up 
					      // a pointer to it after the 
					      // last pointer in object_array
   	*object_array[last_member] = object; //copy the object to our new one
	};
     };
   };

void Set_String_::get_next_member (String& object)
   {
   if (last_accessed_member<last_member)
	{
	object=*object_array[++last_accessed_member];
	};
   };

String Set_String_::get_next_member()
   {
   if (last_accessed_member<last_member)
	return (*object_array[++last_accessed_member]);
   else {String dummy; return dummy;};
   };

void Set_String_::get_first_member (String& object)
   {
   next_member_returns_first();
   get_next_member(object);
   };

String Set_String_::get_first_member ()
   {
   next_member_returns_first();
   return (get_next_member());
   };


void Set_String_::empty()
   {
   for (int n=0; n<=last_member; n++)
	{
	delete(object_array[n]);  // delete a set member
	};
   last_member=-1;
   };  

void Set_String_::shrink()
   {
   if (size==0) // if the set is empty, just free up all allocated memory
	{
	if (object_array!=NULL) {free (object_array); object_array=NULL;};
	}
     else     // free up memory allocated for spare pointer & mv's
	{
	if (object_array!=NULL)
	  object_array=(String**)realloc(object_array, sizeof(String**)*(last_member+1));
	};
   size=last_member+1;
   };

void Set_String_::operator=(const Set_String_& original)
   {
   empty();
   for (int n=0; n<=original.last_member; n++)
	{
	add_member(*original.object_array[n], NO_TEST_FOR_DUPLICATE);
	};
   };


int Set_String_::find_member(const String& possible_member) const
   {
   // this returns -1 if it can't find the member, else the object_array
   // index for the member
   int n=last_member;
   while (n>-1)
	{
//cout <<"C: "<< *object_array[n]<<"  "<<possible_member<<"\n"; 
	if (*object_array[n]==possible_member) break;
	n--;
	};
   return n;
   };
	

void Set_String_::delete_member (const String& object)
   {
   // delete member if we are storing it
   do_delete_member (find_member(object));
   };

void Set_String_::do_delete_member (const int index)
   {
   if (index!=-1)
	{
	delete(object_array[index]);
	object_array[index]=object_array[last_member];
	last_member--;
	};
   };


int Set_String_::has_member(const String& object) 
   {return( (find_member(object)!=-1) ? 1 : 0);};


void Set_String_::operator+= (const Set_String_& set_b)
   {
   //int this_index;
   for (int n=0; n<=set_b.last_member; n++)
	{
	//this_index=find_member(*set_b.object_array[n]);
	//if (this_index==-1)
	     add_member(*set_b.object_array[n]);
	};
   };

void Set_String_::operator-= (const Set_String_& set_b)
   {
   //int this_index;
   for (int n=0; n<=set_b.last_member; n++)
	{
	//this_index=find_member(*set_b.object_array[n]);
	//if (this_index==-1)
	     delete_member(*set_b.object_array[n]);
	};
   };


int Set_String_::operator== (Set_String_& set_b)
  {
  if (no_of_members()!=set_b.no_of_members()) return 0;
  next_member_returns_first();
  set_b.next_member_returns_first();
  while (can_use_get_next_member())
	{
	if ( !set_b.has_member(get_next_member()) ) return 0;
	};
  return 1;
  };

void Set_String_::operator*= (const Set_String_& set_b)
  {
  int b_index;
  next_member_returns_first();
  for (int n=last_member; n>=0; n--)    //NB last_member changing too!
	{
	b_index=set_b.find_member(*object_array[n]);
	if (b_index==-1) 
	     do_delete_member(n);
	};
   };


void Set_String_::screen_dump()  //use for debugging
   {
   cout << '\n';
   for (int n=0; n<=last_member; n++)
	{
	cout << *object_array[n] << "  " << '\n';
	};
   cout << '\n';
   };





void Set_Resource_::fill_with 
	(Resource array[], int start_index, int end_index)
   {
   for (int n=start_index; n<=end_index; n++)
	add_member(array[n]);
   };

Set_Resource_::Set_Resource_ (unsigned expand_rate_=10)
   {
   expand_rate=expand_rate_;
   size=0;
   last_member=-1;
   last_accessed_member=-1;
   object_array=NULL;   // no point in mallocing them memory yet, as the set
			// has no members, so we must NULL them so as not to
			// confuse other member functions, eg empty
   };


Set_Resource_::Set_Resource_ (const Set_Resource_& original)
   {
   expand_rate=original.expand_rate;
   size=0;
   last_member=-1;
   last_accessed_member=-1;
   object_array=NULL;
   operator=(original);  
   };


Set_Resource_::~Set_Resource_ ()  
   {
   for (int n=0; n<=last_member; n++)
	{
	delete(object_array[n]);   // delete memory of a set member
	};
   if (object_array!=NULL) free (object_array); //delete object pointer array
   };


void Set_Resource_::resize_to (unsigned newsize)
   {
   if (newsize>size)   // ie only resize if we need to
     {
     // this if is necessary as some versions of realloc 
     // don't work with NULL pointers
     if (object_array==NULL)  
	object_array = (Resource**) 
			   malloc((newsize+expand_rate+1)*sizeof(Resource**));
      else
	object_array = (Resource**) realloc(object_array, 
				(newsize+expand_rate+1)*sizeof(Resource**) );
      size=newsize+expand_rate+1;
      };
   };

void Set_Resource_::add_member (const Resource& object,
				    const Add_Flag flag = TEST_FOR_DUPLICATE)
   {
   if (flag==NO_TEST_FOR_DUPLICATE)
	{
	resize_to(no_of_members()+1);   // make sure theres room
   	object_array[++last_member] = new(Resource);
					      // make a new object & set up 
					      // a pointer to it after the 
					      // last pointer in object_array
   	*object_array[last_member] = object; //copy the object to our new one
	}
    else
     {
     if (find_member(object)==-1)  // ie can't have duplicates in a set!
	{
	resize_to(no_of_members()+1);   // make sure theres room
   	object_array[++last_member] = new(Resource); 
					      // make a new object & set up 
					      // a pointer to it after the 
					      // last pointer in object_array
   	*object_array[last_member] = object; //copy the object to our new one
	};
     };
   };

void Set_Resource_::get_next_member (Resource& object)
   {
   if (last_accessed_member<last_member)
	{
	object=*object_array[++last_accessed_member];
	};
   };

Resource Set_Resource_::get_next_member()
   {
   if (last_accessed_member<last_member)
	return (*object_array[++last_accessed_member]);
   else {Resource dummy; return dummy;};
   };

void Set_Resource_::get_first_member (Resource& object)
   {
   next_member_returns_first();
   get_next_member(object);
   };

Resource Set_Resource_::get_first_member ()
   {
   next_member_returns_first();
   return (get_next_member());
   };


void Set_Resource_::empty()
   {
   for (int n=0; n<=last_member; n++)
	{
	delete(object_array[n]);  // delete a set member
	};
   last_member=-1;
   };  

void Set_Resource_::shrink()
   {
   if (size==0) // if the set is empty, just free up all allocated memory
	{
	if (object_array!=NULL) {free (object_array); object_array=NULL;};
	}
     else     // free up memory allocated for spare pointer & mv's
	{
	if (object_array!=NULL)
	  object_array=(Resource**)realloc(object_array, sizeof(Resource**)*(last_member+1));
	};
   size=last_member+1;
   };

void Set_Resource_::operator=(const Set_Resource_& original)
   {
   empty();
   for (int n=0; n<=original.last_member; n++)
	{
	add_member(*original.object_array[n], NO_TEST_FOR_DUPLICATE);
	};
   };


int Set_Resource_::find_member(const Resource& possible_member) const
   {
   // this returns -1 if it can't find the member, else the object_array
   // index for the member
   int n=last_member;
   while (n>-1)
	{
	if (*object_array[n]==possible_member) break;
	n--;
	};
   return n;
   };
	

void Set_Resource_::delete_member (const Resource& object)
   {
   // delete member if we are storing it
   do_delete_member (find_member(object));
   };

void Set_Resource_::do_delete_member (const int index)
   {
   if (index!=-1)
	{
	delete(object_array[index]);
	object_array[index]=object_array[last_member];
	last_member--;
	};
   };


int Set_Resource_::has_member(const Resource& object) 
   {return( (find_member(object)!=-1) ? 1 : 0);};


void Set_Resource_::operator+= (const Set_Resource_& set_b)
   {
   //int this_index;
   for (int n=0; n<=set_b.last_member; n++)
	{
	//this_index=find_member(*set_b.object_array[n]);
	//if (this_index==-1)
	     add_member(*set_b.object_array[n]);
	};
   };

void Set_Resource_::operator-= (const Set_Resource_& set_b)
   {
   //int this_index;
   for (int n=0; n<=set_b.last_member; n++)
	{
	//this_index=find_member(*set_b.object_array[n]);
	//if (this_index==-1)
	     delete_member(*set_b.object_array[n]);
	};
   };


void Set_Resource_::operator*= (const Set_Resource_& set_b)
  {
  int b_index;
  next_member_returns_first();
  for (int n=last_member; n>=0; n--)    //NB last_member changing too!
	{
	b_index=set_b.find_member(*object_array[n]);
	if (b_index==-1) 
	     do_delete_member(n);
	};
   };


void Set_Resource_::screen_dump()  //use for debugging
   {
   cout << '\n';
   for (int n=0; n<=last_member; n++)
	{
	cout << (*object_array[n]).url << "  " << '\n';
	cout << (*object_array[n]).name << "  " << '\n';
	//cout << (*object_array[n]).description << "  " << '\n';
	};
   cout << '\n';
   };






void SSByteStore::store(Set_String_& string_set)
   {
   ByteStore::store (string_set.no_of_members());
   string_set.next_member_returns_first();
   for (int n=1; n<=string_set.no_of_members(); n++)
	ByteStore::store(string_set.get_next_member());
   };

void SSByteStore::store(Subject& subject)
   {
   ByteStore::store(subject.subject_id);
   store(subject.urls);
   store(subject.child_subjects);
   store(subject.parent_subjects);
   };

void SSByteStore::store(Record& record)
   {
   ByteStore::store(record.resource.name);
   //ByteStore::store(record.resource.description);
   store(record.subject_ids);
   };

void SSByteStore::get(Set_String_& string_set)
   {
   int members=get_int();
   string_set.empty();
   string_set.resize_to(members);
   String temp;
   for (int n=1; n<=members; n++)
	{
	get(temp);
	string_set.add_member(temp, NO_TEST_FOR_DUPLICATE);
	};
   };

void SSByteStore::get(Subject& subject)
   {
   ByteStore::get(subject.subject_id);
   get(subject.urls);
   get(subject.child_subjects);
   get(subject.parent_subjects);
   };

void SSByteStore::get(Record& record)
   {
   ByteStore::get(record.resource.name);
   //ByteStore::get(record.resource.description);
   get(record.subject_ids);
   };






void Set_Set_String_::fill_with 
	(Set_String_ array[], int start_index, int end_index)
   {
   for (int n=start_index; n<=end_index; n++)
	add_member(array[n]);
   };

Set_Set_String_::Set_Set_String_ (unsigned expand_rate_=10)
   {
   expand_rate=expand_rate_;
   size=0;
   last_member=-1;
   last_accessed_member=-1;
   object_array=NULL;   // no point in mallocing them memory yet, as the set
			// has no members, so we must NULL them so as not to
			// confuse other member functions, eg empty
   };


Set_Set_String_::Set_Set_String_ (const Set_Set_String_& original)
   {
   expand_rate=original.expand_rate;
   size=0;
   last_member=-1;
   last_accessed_member=-1;
   object_array=NULL;
   operator=(original);  
   };


Set_Set_String_::~Set_Set_String_ ()  
   {
   for (int n=0; n<=last_member; n++)
	{
	delete(object_array[n]);   // delete memory of a set member
	};
   if (object_array!=NULL) free (object_array); //delete object pointer array
   };


void Set_Set_String_::resize_to (unsigned newsize)
   {
   if (newsize>size)   // ie only resize if we need to
     {
     // this if is necessary as some versions of realloc 
     // don't work with NULL pointers
     if (object_array==NULL)  
	object_array = (Set_String_**) 
			   malloc((newsize+expand_rate+1)*sizeof(Set_String_**));
      else
	object_array = (Set_String_**) realloc(object_array, 
				(newsize+expand_rate+1)*sizeof(Set_String_**) );
      size=newsize+expand_rate+1;
      };
   };

void Set_Set_String_::add_member (const Set_String_& object,
				    const Add_Flag flag = TEST_FOR_DUPLICATE)
   {
   if (flag==NO_TEST_FOR_DUPLICATE)
	{
	resize_to(no_of_members()+1);   // make sure theres room
   	object_array[++last_member] = new(Set_String_);
					      // make a new object & set up 
					      // a pointer to it after the 
					      // last pointer in object_array
   	*object_array[last_member] = object; //copy the object to our new one
	}
    else
     {
     if (find_member(object)==-1)  // ie can't have duplicates in a set!
	{
	resize_to(no_of_members()+1);   // make sure theres room
   	object_array[++last_member] = new(Set_String_); 
					      // make a new object & set up 
					      // a pointer to it after the 
					      // last pointer in object_array
   	*object_array[last_member] = object; //copy the object to our new one
	};
     };
   };

void Set_Set_String_::get_next_member (Set_String_& object)
   {
   if (last_accessed_member<last_member)
	{
	object=*object_array[++last_accessed_member];
	};
   };

Set_String_ Set_Set_String_::get_next_member()
   {
   if (last_accessed_member<last_member)
	return (*object_array[++last_accessed_member]);
   else {Set_String_ dummy; return dummy;};
   };

void Set_Set_String_::get_first_member (Set_String_& object)
   {
   next_member_returns_first();
   get_next_member(object);
   };

Set_String_ Set_Set_String_::get_first_member ()
   {
   next_member_returns_first();
   return (get_next_member());
   };


void Set_Set_String_::empty()
   {
   for (int n=0; n<=last_member; n++)
	{
	delete(object_array[n]);  // delete a set member
	};
   last_member=-1;
   };  

void Set_Set_String_::shrink()
   {
   if (size==0) // if the set is empty, just free up all allocated memory
	{
	if (object_array!=NULL) {free (object_array); object_array=NULL;};
	}
     else     // free up memory allocated for spare pointer & mv's
	{
	if (object_array!=NULL)
	  object_array=(Set_String_**)realloc(object_array, sizeof(Set_String_**)*(last_member+1));
	};
   size=last_member+1;
   };

void Set_Set_String_::operator=(const Set_Set_String_& original)
   {
   empty();
   for (int n=0; n<=original.last_member; n++)
	{
	add_member(*original.object_array[n], NO_TEST_FOR_DUPLICATE);
	};
   };


int Set_Set_String_::find_member(const Set_String_& possible_member) const
   {
   // this returns -1 if it can't find the member, else the object_array
   // index for the member
   int n=last_member;
   while (n>-1)
	{
	if (*object_array[n]==possible_member) break;
	n--;
	};
   return n;
   };
	

void Set_Set_String_::delete_member (const Set_String_& object)
   {
   // delete member if we are storing it
   do_delete_member (find_member(object));
   };

void Set_Set_String_::do_delete_member (const int index)
   {
   if (index!=-1)
	{
	delete(object_array[index]);
	object_array[index]=object_array[last_member];
	last_member--;
	};
   };


int Set_Set_String_::has_member(const Set_String_& object) 
   {return( (find_member(object)!=-1) ? 1 : 0);};


void Set_Set_String_::operator+= (const Set_Set_String_& set_b)
   {
   //int this_index;
   for (int n=0; n<=set_b.last_member; n++)
	{
	//this_index=find_member(*set_b.object_array[n]);
	//if (this_index==-1)
	     add_member(*set_b.object_array[n]);
	};
   };

void Set_Set_String_::operator-= (const Set_Set_String_& set_b)
   {
   //int this_index;
   for (int n=0; n<=set_b.last_member; n++)
	{
	//this_index=find_member(*set_b.object_array[n]);
	//if (this_index==-1)
	     delete_member(*set_b.object_array[n]);
	};
   };


void Set_Set_String_::operator*= (const Set_Set_String_& set_b)
  {
  int b_index;
  next_member_returns_first();
  for (int n=last_member; n>=0; n--)    //NB last_member changing too!
	{
	b_index=set_b.find_member(*object_array[n]);
	if (b_index==-1) 
	     do_delete_member(n);
	};
   };


void Set_Set_String_::screen_dump()  //use for debugging
   {
   cout << '\n';
   for (int n=0; n<=last_member; n++)
	{
	(*object_array[n]).screen_dump();
	};
   cout << '\n';
   };


