
#ifndef _ss_h
#define _ss_h 1


#include <String.h>
#include "ByteStore.h"
#include "functions.h"

enum Add_Flag { TEST_FOR_DUPLICATE, NO_TEST_FOR_DUPLICATE };


class Subject;
class Resource;
class Record;

// replacements for Set<T>


class Set_String_
   {
   public:

   //usual stuff for dynamic class:

	Set_String_ (unsigned expand_rate_=10);  // contructor
	~Set_String_ ();   // destructor
	Set_String_ (const Set_String_& original);  // copy constructor
	void operator=(const Set_String_& original);  

   //usual stuff for container classes:

	void add_member (const String& object, 
				const Add_Flag flag = TEST_FOR_DUPLICATE);
	int has_member(const String& object); // 1=true 0=flase
	void delete_member (const String& object); //beware, this cause an order
					      	   //change
	void empty(); //deletes all members

   //use these to get objects out, use a for loop with no_of_members(),
   //or a while loop with can_get_next_member().
   //always call next_member_returns_first just before the loop, to reset
   //the internal itorator, so get_next_member starts with the first member

	int no_of_members (void) const {return last_member+1;};
	int can_use_get_next_member (void) const
		{return (last_accessed_member<last_member);};
	void next_member_returns_first(void)
        	{last_accessed_member=-1;};   // doesn't change members
	String get_first_member (void); // doesn't change members
	String get_next_member (void);  // doesn't change members
	void get_next_member (String& object);
	void get_first_member (String& object);

	int operator== (Set_String_& set_b);
	void operator+= (const Set_String_& set_b);   //union_ with assign
	void operator*= (const Set_String_& set_b);   //intersect with assign
	void operator-= (const Set_String_& set_b);   //difference with assign
	Set_String_ operator+ (const Set_String_& set_b)
		{Set_String_ result=(*this); result+=set_b; return result;};
	Set_String_ operator* (const Set_String_& set_b)
		{Set_String_ result=(*this); result*=set_b; return result;};
	Set_String_ operator- (const Set_String_& set_b)
		{Set_String_ result=(*this); result-=set_b; return result;};


   //for memory management - ignore it if you like, but the class won't 
   //bother deallocating until its destructor is called

	void shrink();  // deallocates all memory for spare object slots
	void resize_to (unsigned newsize);    // to make it bigger, or
		//smaller, but you can't use it as a short cut to 
		//deleting members
	int allocated_size() {return size;}; // how many objects it has room
					     // for in total

   //usefull
	void screen_dump();  //use for debugging
	void fill_with (String array[], int start_index, int end_index);


   protected:
	void do_delete_member (const int index);
	int find_member(const String& possible_member) const;
	unsigned expand_rate;
	int size; 
	int last_member; 
	int last_accessed_member;
	String** object_array;
   };

typedef Set_String_ Node_id; //Node_id is a set of subject_id's


class Set_Resource_
   {
   public:

   //usual stuff for dynamic class:

	Set_Resource_ (unsigned expand_rate_=10);  // contructor
	~Set_Resource_ ();   // destructor
	Set_Resource_ (const Set_Resource_& original);  // copy constructor
	void operator=(const Set_Resource_& original);  

   //usual stuff for container classes:

	void add_member (const Resource& object,
				 const Add_Flag flag = TEST_FOR_DUPLICATE);
	int has_member(const Resource& object); // 1=true 0=flase
	void delete_member (const Resource& object); // beware, this cause an order
					      //change
	void empty(); //deletes all members

   //use these to get objects out, use a for loop with no_of_members(),
   //or a while loop with can_get_next_member().
   //always call next_member_returns_first just before the loop, to reset
   //the internal itorator, so get_next_member starts with the first member

	int no_of_members (void) const {return last_member+1;};
	int can_use_get_next_member (void) const
		{return (last_accessed_member<last_member);};
	void next_member_returns_first(void)
        	{last_accessed_member=-1;};   // doesn't change members
	Resource get_first_member (void); // doesn't change members
	Resource get_next_member (void);  // doesn't change members
	void get_next_member (Resource& object);
	void get_first_member (Resource& object);

	void operator+= (const Set_Resource_& set_b);   //union_ with assign
	void operator*= (const Set_Resource_& set_b);   //intersect with assign
	void operator-= (const Set_Resource_& set_b);   //difference with assign
	Set_Resource_ operator+ (const Set_Resource_& set_b)
		{Set_Resource_ result=(*this); result+=set_b; return result;};
	Set_Resource_ operator* (const Set_Resource_& set_b)
		{Set_Resource_ result=(*this); result*=set_b; return result;};
	Set_Resource_ operator- (const Set_Resource_& set_b)
		{Set_Resource_ result=(*this); result-=set_b; return result;};


   //for memory management - ignore it if you like, but the class won't 
   //bother deallocating until its destructor is called

	void shrink();  // deallocates all memory for spare object slots
	void resize_to (unsigned newsize);    // to make it bigger, or
		//smaller, but you can't use it as a short cut to 
		//deleting members
	int allocated_size() {return size;}; // how many objects it has room
					     // for in total

   //usefull
	void screen_dump();  //use for debugging
	void fill_with (Resource array[], int start_index, int end_index);


   protected:
	void do_delete_member (const int index);
	int find_member(const Resource& possible_member) const;
	unsigned expand_rate;
	int size; 
	int last_member; 
	int last_accessed_member;
	Resource** object_array;
   };




class Subject
   {
   public:
	String subject_id;
	Set_String_ urls;
	Set_String_ child_subjects;
	Set_String_ parent_subjects;
   };


class Resource
   {
   public: 
	String url;
	String name;
	//String description;
	void fill_with (String array[], int start_index)
		{
		int i=start_index;
		url=array[i++];
		name=array[i++];
		//description=array[i++];
		}; 
	int operator== (const Resource& res_b)
		{return url==res_b.url;};
	void operator= (const Resource& orig)
		{url=orig.url; name=orig.name; 
			/*description=orig.description;*/ };	
   };


class Record
   {
   public:
	Resource resource;
	Set_String_ subject_ids;
	Record() {};
	Record (Resource resource_, Set_String_ subject_ids_) 
		{resource=resource_; subject_ids=subject_ids_;};
   };



/*
class RelatedSubject
   {
   public: 
	String subject_id;
	int no_of_common_resources;
	//int link_use; 
	//int link_agreement;
   	int operator!= (const RelatedSubject& other);
	RelatedSubject(const RelatedSubject& original)
		{no_of_common_resources=original.no_of_common_resources;
		 subject_id=original.subject_id;};
	RelatedSubject() {};

	void operator=(const RelatedSubject& rs)
		{subject_id=rs.subject_id;
		 no_of_common_resources=rs.no_of_common_resources;};
   };

int RelatedSubject::operator!= (const RelatedSubject& other)
	{
	return (subject_id!=other.subject_id);
	};

int Set_RelatedSubject_::find_member(const RelatedSubject& possible_member)
 									const
  //returns the member number or -1
   {
   int n=last_member;
   while ( n!=(-1) &&
	     ((*object_array[n]).subject_id!=possible_member.subject_id))
	 {n--;};
   return n;
   };
*/


class SSByteStore : public ByteStore
   {
   public:
	SSByteStore(int start_size, int expand_by_)
	  :ByteStore(start_size, expand_by_) {;}; //just calls base constructor
	void store(Set_String_& string_set);
	void store(Subject& subject);
	void store(Record& record);
	void get(Set_String_& string_set);
	void get(Subject& subject);
	void get(Record& record);

	// these are needed as the fns above don't overload base fns,
	// they hide them.
	void store(String& string)
		{ByteStore::store(string);};
	void get(String& string)
		{ByteStore::get(string);};
	
   };


void string_set_to_string(Set_String_ set, String& string);

void resource_set_to_string(Set_Resource_ set, String& string);



class Set_Set_String_
   {
   public:

   //usual stuff for dynamic class:

	Set_Set_String_ (unsigned expand_rate_=10);  // contructor
	~Set_Set_String_ ();   // destructor
	Set_Set_String_ (const Set_Set_String_& original);  // copy constructor
	void operator=(const Set_Set_String_& original);  

   //usual stuff for container classes:

	void add_member (const Set_String_& object,
				 const Add_Flag flag = TEST_FOR_DUPLICATE);
	int has_member(const Set_String_& object); // 1=true 0=flase
	void delete_member (const Set_String_& object); // beware, this cause an order
					      //change
	void empty(); //deletes all members

   //use these to get objects out, use a for loop with no_of_members(),
   //or a while loop with can_get_next_member().
   //always call next_member_returns_first just before the loop, to reset
   //the internal itorator, so get_next_member starts with the first member

	int no_of_members (void) const {return last_member+1;};
	int can_use_get_next_member (void) const
		{return (last_accessed_member<last_member);};
	void next_member_returns_first(void)
        	{last_accessed_member=-1;};   // doesn't change members
	Set_String_ get_first_member (void); // doesn't change members
	Set_String_ get_next_member (void);  // doesn't change members
	void get_next_member (Set_String_& object);
	void get_first_member (Set_String_& object);

	void operator+= (const Set_Set_String_& set_b);   //union_ with assign
	void operator*= (const Set_Set_String_& set_b);   //intersect with assign
	void operator-= (const Set_Set_String_& set_b);   //difference with assign
	Set_Set_String_ operator+ (const Set_Set_String_& set_b)
		{Set_Set_String_ result=(*this); result+=set_b; return result;};
	Set_Set_String_ operator* (const Set_Set_String_& set_b)
		{Set_Set_String_ result=(*this); result*=set_b; return result;};
	Set_Set_String_ operator- (const Set_Set_String_& set_b)
		{Set_Set_String_ result=(*this); result-=set_b; return result;};


   //for memory management - ignore it if you like, but the class won't 
   //bother deallocating until its destructor is called

	void shrink();  // deallocates all memory for spare object slots
	void resize_to (unsigned newsize);    // to make it bigger, or
		//smaller, but you can't use it as a short cut to 
		//deleting members
	int allocated_size() {return size;}; // how many objects it has room
					     // for in total

   //usefull
	void screen_dump();  //use for debugging
	void fill_with (Set_String_ array[], int start_index, int end_index);


   protected:
	void do_delete_member (const int index);
	int find_member(const Set_String_& possible_member) const;
	unsigned expand_rate;
	int size; 
	int last_member; 
	int last_accessed_member;
	Set_String_** object_array;
   };

typedef Set_Set_String_ Set_Node_id_; //Node_id is a set of subject_id's



#endif
