#include <iostream.h>
#include <gdbm.h>
#include <stdio.h>
#include "ssdbm.h"
#include "ss.h"
#include "ssci-lang.h"

//****************************************************************************
// IMPORTANT -- THESE ARE INPUT COMMAND SIZE LIMITS ---
//****************************************************************************

const BUFSIZE = 4000; // max size of request in chars
String strings[500];	// this should be in service_request, but the compiler
			// doesn't like it there!  :-(

//*****************************************************************************


String string_token;  // used to pass the value of strings back from the lexer
char* flex_input_ptr;     // flex reads from here, and advances this ptr
			  // NB flex must read a '\n' before it stops!!!

SS_Error service_request (SS_Map_Manager& ssmm, char* request, String& result,
						    String& message);

extern int yylex();  // declaration of lexer - yylex() will actualy return
		     // an SS_Token 



int main(int argc, char* argv[])
   {
   if (argc!=2) cout <<"arg 1 = database name \n";
   else 
     {
     SS_Error error;
     int file_mode;
     char in_buffer[BUFSIZE];
     SS_Map_Manager ssmm(error, argv[1], GDBM_WRCREAT);
     if (error==OK)
       {
       if (setvbuf(stdin, NULL, _IOLBF, 0) != 0)
		{cerr<<"setvbuf error\n"; exit -1;};
       while (fgets(in_buffer, BUFSIZE, stdin) != NULL)
	{
        String result;
	String message;
	error=service_request(ssmm, in_buffer, result, message);
	if (result!="")
	  {
	  if (result.lastchar()!='\n') result+='\n';
	  while (result.elem(result.length()-2)==' ') 
			result.del(result.length()-2,1);
	  cout << result;
	  }
	 else cout << '\n';
	cout.flush();
	if (message!="") 
	  {
	  cerr <<argv[0]<<": "<<message<<" : "<<in_buffer<<'\n';
	  cerr.flush();
	  };
	}
       }
      else
	{cerr<<argv[0]<< " error opening subject space - can only open subject space by one program at a time\n";};
    };
   };




SS_Error service_request 
		(SS_Map_Manager& ssmm, char* request_ptr, String& result,
							String& message)
 // request_ptr must point into a char buffer ending in '\n'
   {
   flex_input_ptr=request_ptr;
   SS_Error error;

   // find out what the service request is
   SS_Token service = (SS_Token) yylex();
   // parse the request data, an unknown number of strings
   int last_string=-1;

   SS_Token token = service;
   while ((token!=EOL)) 
	{
	token = (SS_Token) yylex();
	if (token==STRING) strings[++last_string]=string_token;
	};

   int no_of_strings=last_string+1;   

   // process the service request
   switch (service)
     {
     case SUBJECTS_LIST:
	{
	if (no_of_strings==0)
	  {
	  Set_String_ subject_ids;
	  error = ssmm.subjects_list(subject_ids);
	  if (error==OK) string_set_to_string(subject_ids, result);
	  }
	 else error=UNRECOGNISED_REQUEST;
	break;
	}
     case INTERSECTORS_OF_SUBJECT:
	{
	if (no_of_strings==1)
	  {
	  Set_String_ subject_ids;
	  error = ssmm.intersectors_of_subject(strings[0], subject_ids);
	  if (error==OK) string_set_to_string(subject_ids, result);
	  }
	 else error=UNRECOGNISED_REQUEST;
	break;
	}
     case NEIGHBOR_NODES:
	{
	if (no_of_strings>=1)
	  {
	  Set_Node_id_ neighbors;
	  Node_id node_id;
	  node_id.fill_with(strings, 0, last_string);
	  error=ssmm.neighbor_nodes(node_id, neighbors);
	  if (error==OK) 
	    {
	    String temp;
	    neighbors.next_member_returns_first();
	    while (neighbors.can_use_get_next_member())
		{
		string_set_to_string
			(neighbors.get_next_member(), temp);
		result += temp;
		result += ",";
		};
	    };
	  }
	 else error=UNRECOGNISED_REQUEST;
	break;
	}
     case URLS_IN_INTERSECTION:
	{
	if (no_of_strings>=1)
	  {
	  Set_String_ urls;
	  Set_String_ subject_ids;
	  subject_ids.fill_with(strings, 0, last_string);
	  error = ssmm.urls_in_intersection(subject_ids, urls);
	  if (error==OK) string_set_to_string(urls, result);
	  }
	 else error=UNRECOGNISED_REQUEST;
	break;
	}
     case RESOURCES_IN_NODE:
	{
	if (no_of_strings>=1)
	  {
	  Set_Resource_ resources;
	  Set_String_ node_id;
	  node_id.fill_with(strings, 0, last_string);
	  error = ssmm.resources_in_node(node_id, resources);
	  if (error==OK) resource_set_to_string(resources, result);
	  }
	 else error=UNRECOGNISED_REQUEST;
	break;
	}
     case TEXT_DUMP:
	{
	ssmm.text_dump(result);
	error=OK;
	break;
	}

     default:
	{
	error=UNRECOGNISED_REQUEST;
	};
     }; //end switch

   switch (error)
	{
	case OK: 
		{message=""; break;}
	case UNRECOGNISED_REQUEST: 
		{message="request not understood"; break;}
	case ILEGAL_REQUEST: 
		{message="ilegal request"; break;}
	case RESOURCE_NOT_FOUND: 
		{message="resource not found"; break;}
	case SUBJECT_NOT_FOUND: 
		{message="subject not found"; break;}
	case RESOURCE_ALREADY_EXISTS: 
		{message="resource already extists"; break;}
	case SUBJECT_ALREADY_EXISTS: 
		{message="subject already exists"; break;}
	case NO_SUCH_NODE: 
		{message="no such node"; break;}
	default: message="error";
	}; 
   return error;
   };




