#include <iostream.h>
#include <gdbm.h>
#include <stdio.h>
#include "ssdbm.h"
#include "ss.h"
#include "ssci-lang.h"

//****************************************************************************
// IMPORTANT -- THESE ARE INPUT COMMAND SIZE LIMITS ---
//****************************************************************************

const BUFSIZE = 4000; // max size of request in chars
String strings[500];	// this should be in service_request, but the compiler
			// doesn't like it there!  :-(

//*****************************************************************************


String string_token;  // used to pass the value of strings back from the lexer
char* flex_input_ptr;     // flex reads from here, and advances this ptr
			  // NB flex must read a '\n' before it stops!!!


SS_Error service_request (SSDBM& sd, char* request, String& result,
						    String& message);

extern int yylex();  // declaration of lexer - yylex() will actualy return
		     // an SS_Token 



int main(int argc, char* argv[])
   {
   if (argc!=2) cout <<"arg 1 = database name \n";
   else 
     {
     SS_Error error;
     int file_mode;
     char in_buffer[BUFSIZE];
     SSDBM ssdbm(error, argv[1], GDBM_WRCREAT);
     if (error==OK)
       {
       if (setvbuf(stdin, NULL, _IOLBF, 0) != 0)
		{cerr<<"setvbuf error\n"; exit -1;};
       while (fgets(in_buffer, BUFSIZE, stdin) != NULL)
	{
        String result;
	String message;
	error=service_request(ssdbm, in_buffer, result, message);
	if (result!="")
	  {
	  if (result.lastchar()!='\n') result+='\n';
	  while (result.elem(result.length()-2)==' ') 
			result.del(result.length()-2,1);
	  cout << result;
	  }
	 else cout << '\n';
	cout.flush();
	if (message!="") 
	  {
	  cerr <<argv[0]<<": "<<message<<" : "<<in_buffer<<'\n';
	  cerr.flush();
	  };
	}
       }
      else
	{cerr<<argv[0]<< " error opening subject space - can only open subject space by one program at a time\n";};
    };
   };



SS_Error service_request (SSDBM& sd, char* request_ptr, String& result,
							String& message)
 // request_ptr must point into a char buffer ending in '\n'
   {
   flex_input_ptr=request_ptr;
   SS_Error error;

   // find out what the service request is
   SS_Token service = (SS_Token) yylex();
   // parse the request data, an unknown number of strings
   int last_string=-1;

   SS_Token token = service;
   while ((token!=EOL)) 
	{
	token = (SS_Token) yylex();
	if (token==STRING) strings[++last_string]=string_token;
	};

   int no_of_strings=last_string+1;   

   // process the service request
   switch (service)
     {
     case NEW_RESOURCE:
	{
	if (no_of_strings>=3)
	  {
	  Resource resource;
	  resource.fill_with(strings, 0);
	  Set_String_ subject_ids;
	  subject_ids.fill_with(strings, 2, last_string);
	  error = sd.new_resource (resource, subject_ids);
	  }
	else error=UNRECOGNISED_REQUEST;
	break;
	}; 
     case GET_RESOURCE:
	{
	if (no_of_strings==1)
	  {
	  Resource resource;
	  String url=strings[0];
	  error = sd.get_resource(url, resource);
	  if (error==OK) 
		{
		result+="\""; result+=resource.url; result+="\" ";
		result+="\""; result+=resource.name; result+="\"";
		};
	  }
	 else error=UNRECOGNISED_REQUEST;
	break;
	}
     case UPDATE_RESOURCE:
	{
	if (no_of_strings==2)
	  {
	  Resource resource;
	  resource.fill_with(strings, 0);
	  error = sd.update_resource (resource);
	  }
	else error=UNRECOGNISED_REQUEST;
	break;
	}
     case DELETE_RESOURCE:
	{
	if (no_of_strings==1)
	  {
	  String url = strings[0];
	  error = sd.delete_resource_from_all_subjects(url);
	  }
	else error=UNRECOGNISED_REQUEST;
	break;
	}
     case NEW_SUBJECT:
	{
	if (no_of_strings==1)
	  error = sd.new_subject(strings[0], "");
	else if (no_of_strings==2)
	  error = sd.new_subject(strings[0], strings[1]);
	else error=UNRECOGNISED_REQUEST;
	break;
	}
     case ADD_SUBJECT:
	{
	if (no_of_strings>=2)
	  {
	  Set_String_ parent_ids;
	  parent_ids.fill_with(strings, 1, last_string);
 	  error = sd.add_subject(strings[0], parent_ids);
	  }
	 else error = UNRECOGNISED_REQUEST;
	break;	
	}
     case DELETE_SUBJECT:
	{
	if (no_of_strings==1)
	  error = sd.delete_subject(strings[0]);
	 else error = UNRECOGNISED_REQUEST;
	break;
	}
     case REMOVE_SUBJECT:
	{
	if (no_of_strings==2)
	  error = sd.remove_subject(strings[0], strings[1]);
	 else error = UNRECOGNISED_REQUEST;
	break;	
	}
     case ADD_RESOURCE:
	{
	if (no_of_strings>=2)
	  {
	  Set_String_ subject_ids;
	  subject_ids.fill_with(strings, 1, last_string);
	  error = sd.add_resource_to_subjects(strings[0], subject_ids);
	  }
	else error=UNRECOGNISED_REQUEST;
	break;
	}
     case SET_SUBJECTS_OF_RESOURCE:
	{
	if (no_of_strings>=2)
	  {
	  Set_String_ subject_ids;
	  subject_ids.fill_with(strings, 1, last_string);
	  error = sd.set_subjects_of_resource(strings[0], subject_ids);
	  }
	else error=UNRECOGNISED_REQUEST;
	break;
	}
     case REMOVE_RESOURCE:
	{
	if (no_of_strings>=2)
	  {
	  Set_String_ subject_ids;
	  subject_ids.fill_with(strings, 1, last_string);
	  error = sd.remove_resource_from_subjects(strings[0], subject_ids);
	  }
	else error=UNRECOGNISED_REQUEST;
	break;
	}
     case URLS_IN_INTERSECTION:
	{
	if (no_of_strings>=1)
	  {
	  Set_String_ urls;
	  Set_String_ subject_ids;
	  subject_ids.fill_with(strings, 0, last_string);
	  error = sd.urls_in_intersection(subject_ids, urls);
	  if (error==OK) string_set_to_string(urls, result);
	  }
	 else error=UNRECOGNISED_REQUEST;
	break;
	}
     case RESOURCES_IN_INTERSECTION:
	{
	if (no_of_strings>=1)
	  {
	  Set_Resource_ resources;
	  Set_String_ subject_ids;
	  subject_ids.fill_with(strings, 0, last_string);
	  error = sd.resources_in_intersection(subject_ids, resources);
	  if (error==OK) resource_set_to_string(resources, result);
	  }
	 else error=UNRECOGNISED_REQUEST;
	break;
	}
     case SUBJECTS_OF_RESOURCE:
	{
	if (no_of_strings==1)
	  {
	  Set_String_ subject_ids;
	  error = sd.subjects_of_resource(strings[0], subject_ids);
	  if (error==OK) string_set_to_string(subject_ids, result);
	  }
	 else error=UNRECOGNISED_REQUEST;
	break;  
	} 
     case PARENTS_OF_SUBJECT:
	{
	if (no_of_strings==1)
	  {
	  Set_String_ subject_ids;
	  error = sd.parents_of_subject(strings[0], subject_ids);
	  if (error==OK) string_set_to_string(subject_ids, result);
	  }
	 else error=UNRECOGNISED_REQUEST;
	break;
	}
     case CHILDREN_OF_SUBJECT:
	{
	if (no_of_strings==1)
	  {
	  Set_String_ subject_ids;
	  error = sd.children_of_subject(strings[0], subject_ids);
	  if (error==OK) string_set_to_string(subject_ids, result);
	  }
	 else error=UNRECOGNISED_REQUEST;
	break;
	}
     case INTERSECTORS_OF_SUBJECT:
	{
	if (no_of_strings==1)
	  {
	  Set_String_ subject_ids;
	  error = sd.intersectors_of_subject(strings[0], subject_ids);
	  if (error==OK) string_set_to_string(subject_ids, result);
	  }
	 else error=UNRECOGNISED_REQUEST;
	break;
	}
     case SUBJECTS_LIST:
	{
	if (no_of_strings==0)
	  {
	  Set_String_ subject_ids;
	  error = sd.subjects_list(subject_ids);
	  if (error==OK) string_set_to_string(subject_ids, result);
	  }
	 else error=UNRECOGNISED_REQUEST;
	break;
	}
     case URLS_LIST:
	{
	if (no_of_strings==0)
	  {
	  Set_String_ urls;
	  error = sd.urls_list(urls);
	  if (error==OK) string_set_to_string(urls, result);
	  }
	 else error=UNRECOGNISED_REQUEST;
	break;
	}
     case REORGANISE:
	{
	sd.reorganise();
	error=OK;
	break;
	}
     case TEXT_DUMP:
	{
	sd.text_dump(result);
	error=OK;
	break;
	}

     default:
	{
	error=UNRECOGNISED_REQUEST;
	};
     }; //end switch

   switch (error)
	{
	case OK: 
		{message=""; break;}
	case UNRECOGNISED_REQUEST: 
		{message="request not understood"; break;}
	case ILEGAL_REQUEST: 
		{message="ilegal request"; break;}
	case RESOURCE_NOT_FOUND: 
		{message="resource not found"; break;}
	case SUBJECT_NOT_FOUND: 
		{message="subject not found"; break;}
	case RESOURCE_ALREADY_EXISTS: 
		{message="resource already extists"; break;}
	case SUBJECT_ALREADY_EXISTS: 
		{message="subject already exists"; break;}
	default: message="error";
	}; 
   return error;
   };




