#include "gdbm.h"
#include "ssdbm.h"
#include "ss.h"
#include <tcl.h>
#include "ssdbm-tcl.h"



int ssdbm_resources_in_intersection 
	(ClientData clientData, Tcl_Interp* interp, int argc, char* argv[])
   {
  // arg[1] is the urls list variable name,
  // arg[2] is the resource list var name,
  // arg[3] is the intersection
   Set_Resource_ resources;
   Set_String_ intersection;
   fill_string_set_with_tcl_list (intersection, argv[3]);
   (*ssdbm_ptr).resources_in_intersection(intersection, resources);
   Resource tmp;
   Tcl_SetVar(interp, argv[1], "", 0);  //empty the list
   Tcl_SetVar(interp, argv[2], "", 0);  //empty the list
   while (resources.can_use_get_next_member())
     {
     resources.get_next_member(tmp);
     Tcl_SetVar(interp, argv[1], "{", TCL_APPEND_VALUE);
     Tcl_SetVar(interp, argv[1], tmp.url, TCL_APPEND_VALUE);
     Tcl_SetVar(interp, argv[1], "}", TCL_APPEND_VALUE);
     Tcl_SetVar(interp, argv[1], " ", TCL_APPEND_VALUE);
     Tcl_SetVar(interp, argv[2], "{", TCL_APPEND_VALUE);
     Tcl_SetVar(interp, argv[2], tmp.name, TCL_APPEND_VALUE);
     Tcl_SetVar(interp, argv[2], "}", TCL_APPEND_VALUE);
     Tcl_SetVar(interp, argv[2], " ", TCL_APPEND_VALUE);
     };
   return TCL_OK;
   };

int ssdbm_subjects_list 
	(ClientData clientData, Tcl_Interp* interp, int argc, char* argv[])
   {
  // argv[1] is subjects_list (to be filled)
   Set_String_ subjects;
   (*ssdbm_ptr).subjects_list(subjects);
   String temp;
   subject_id_set_to_tcl_list(interp, subjects, argv[1]);
   return TCL_OK;
   };

int ssdbm_subjects_of_resource 
  	(ClientData clientData, Tcl_Interp* interp, int argc, char* argv[]) 
  // argv[1] is url
  // argv[2] is name of subjects_list
   {
   Set_String_ subjects;
   (*ssdbm_ptr).subjects_of_resource(argv[1], subjects);
   subject_id_set_to_tcl_list (interp, subjects, argv[2]);
   return TCL_OK;
   };

int ssdbm_set_subjects_of_resource 
	(ClientData clientData, Tcl_Interp* interp, int argc, char* argv[])
   {
  // argv[1] is url
  // argv[2] is name of subjects list
   Set_String_ subjects;
   fill_string_set_with_tcl_list (subjects, argv[2]);
   (*ssdbm_ptr).set_subjects_of_resource(argv[1], subjects);
   return TCL_OK;
   };


int ssdbm_new_resource
	(ClientData clientData, Tcl_Interp* interp, int argc, char* argv[])
  // argv[1] is url
  // argv[2] is name
  // argv[3] is a list of subject ids
   {
   Resource resource;
   resource.url=argv[1];
   resource.name=argv[2];
   Set_String_ subject_ids;
   fill_string_set_with_tcl_list (subject_ids, argv[3]);
   (*ssdbm_ptr).new_resource (resource, subject_ids);
   return TCL_OK;
   };

int ssdbm_update_resource
	(ClientData clientData, Tcl_Interp* interp, int argc, char* argv[])
   {
  // argv[1] is url
  // argv[2] is resource name
   Resource resource;
   resource.url=argv[1];
   resource.name=argv[2];
   (*ssdbm_ptr).update_resource (resource);
   return TCL_OK;
   };

int ssdbm_exists_resource
	(ClientData clientData, Tcl_Interp* interp, int argc, char* argv[])
  // argv[1] is a url
  // argv[2] is name of status variable, fill with "yes" or "no"
   {
   if (((*ssdbm_ptr).exists_resource(argv[1]))==RESOURCE_ALREADY_EXISTS)
	  Tcl_SetVar(interp, argv[2], "yes", 0);
     else Tcl_SetVar(interp, argv[2], "no", 0);  
   return TCL_OK;
   };

int ssdbm_delete_resource
	(ClientData clientData, Tcl_Interp* interp, int argc, char* argv[])
  // argv[1] is a url
   {
   (*ssdbm_ptr).delete_resource_from_all_subjects(argv[1]);
   return TCL_OK;
   };

int ssdbm_destructor
	(ClientData clientData, Tcl_Interp* interp, int argc, char* argv[])
  // no args
   {
   (*ssdbm_ptr).~SSDBM();
   return TCL_OK;
   };

int ssdbm_resources_list 
	(ClientData clientData, Tcl_Interp* interp, int argc, char* argv[])
  // argv[1] is the name of the urls list to fill
  // argv[2] is the name of the resources (names) list to fill
   {
   Set_String_ urls;
   Set_String_ resource_names;
   (*ssdbm_ptr).urls_list(urls);
   String_Set_to_tcl_list (interp, urls, argv[1]);
   Resource resource;
   urls.next_member_returns_first();
   while (urls.can_use_get_next_member())
	{
	(*ssdbm_ptr).get_resource (urls.get_next_member(), resource);
	resource_names.add_member(resource.name);
	};
   String_Set_to_tcl_list (interp, resource_names, argv[2]);
   return TCL_OK;
   };


void register_new_tcl_commands (Tcl_Interp* interp)
   {
   Tcl_CreateCommand (interp, "ssdbm_resources_in_intersection", 
			ssdbm_resources_in_intersection, (ClientData *) NULL,
			(Tcl_CmdDeleteProc *) NULL);
   Tcl_CreateCommand (interp, "ssdbm_subjects_list", 
			ssdbm_subjects_list, (ClientData *) NULL,
			(Tcl_CmdDeleteProc *) NULL);
   Tcl_CreateCommand (interp, "ssdbm_update_resource", 
			ssdbm_update_resource, (ClientData *) NULL,
			(Tcl_CmdDeleteProc *) NULL);
   Tcl_CreateCommand (interp, "ssdbm_set_subjects_of_resource", 
			ssdbm_set_subjects_of_resource, (ClientData *) NULL,
			(Tcl_CmdDeleteProc *) NULL);
   Tcl_CreateCommand (interp, "ssdbm_subjects_of_resource", 
			ssdbm_subjects_of_resource, (ClientData *) NULL,
			(Tcl_CmdDeleteProc *) NULL);
   Tcl_CreateCommand (interp, "ssdbm_new_resource", 
			ssdbm_new_resource, (ClientData *) NULL,
			(Tcl_CmdDeleteProc *) NULL);
   Tcl_CreateCommand (interp, "ssdbm_exists_resource", 
			ssdbm_exists_resource, (ClientData *) NULL,
			(Tcl_CmdDeleteProc *) NULL);
   Tcl_CreateCommand (interp, "ssdbm_destructor", 
			ssdbm_destructor, (ClientData *) NULL,
			(Tcl_CmdDeleteProc *) NULL);
   Tcl_CreateCommand (interp, "ssdbm_delete_resource", 
			ssdbm_delete_resource, (ClientData *) NULL,
			(Tcl_CmdDeleteProc *) NULL);
   Tcl_CreateCommand (interp, "ssdbm_resources_list", 
			ssdbm_resources_list, (ClientData *) NULL,
			(Tcl_CmdDeleteProc *) NULL);
   };   


void fill_string_set_with_tcl_list (Set_String_& intersection, char* list)
   {
   char* ptr = list;
   while ((*ptr)!='\0')
	{
	//skip whitespace
	String tmp;
	while ((*ptr=='\t')||(*ptr=='\x020')) ptr++; // \x020 is SPACE char 
	while ((*ptr!='\0')&&(*ptr!='\t')&&(*ptr!='\x020'))
		{
		tmp += *ptr;
		ptr++;
		};
	if (tmp.length()>0) 
		intersection.add_member(tmp);
	};
   };



void Resource_to_tcl_list_element (Resource resource, String string)
   {
   string = "{";
   string += resource.url;
   string += " ";
   string += resource.name;
   string += "}";
   };

void String_Set_to_tcl_list 
		(Tcl_Interp* interp, Set_String_& set, char* listname)
   {
   Tcl_SetVar(interp, listname, "" , 0); //empty list
   String temp;
   while (set.can_use_get_next_member())
     {
     temp = set.get_next_member();
     Tcl_SetVar(interp, listname, "{", TCL_APPEND_VALUE);
     Tcl_SetVar(interp, listname, temp, TCL_APPEND_VALUE);
     Tcl_SetVar(interp, listname, "}", TCL_APPEND_VALUE);
     Tcl_SetVar(interp, listname, " ", TCL_APPEND_VALUE);
     }; 
   };

void subject_id_set_to_tcl_list 
		(Tcl_Interp* interp, Set_String_& set, char* listname)
   {
   Tcl_SetVar(interp, listname, "" , 0); //empty list
   String temp;
   while (set.can_use_get_next_member())
     {
     temp = set.get_next_member();
     Tcl_SetVar(interp, listname, temp, TCL_APPEND_VALUE);
     Tcl_SetVar(interp, listname, " ", TCL_APPEND_VALUE);
     }; 
   };
